// PE43703_CONTROLLER_V1-0 30 JAN 2025

#include "Wire.h"
#include <EEPROM.h>              // stores dB value for power up
#include "SSD1306Ascii.h"
#include "SSD1306AsciiWire.h"
#define I2C_ADDRESS 0x3C         // 0x3C or 0x3D  OLED
SSD1306AsciiWire oled;

String callSign = "PE43703 ";   // 8 chars max including spaces

    // IO assignment
const byte SW_UP =  6;  // input SW1 high = PB not pushed
const byte SW_DN = 11;  // input SW2

const byte LE  = A0;  // output Latch Enable
const byte CLK = A1;  // output Clock
const byte DTA = A2;  // output Data

bool SW_UP_L;  //LATCH pushbutton
bool SW_DN_L;
bool SW_UP_S;  //STORE last state for edge detection
bool SW_DN_S;
bool SW_UP_R;  //READ pushbuttom from IO
bool SW_DN_R;
byte SWtmr; //repeat timer
const byte SWtmA = 75; // delay to 1st repeat x 10 mS
const byte SWtmB = 10; // delay to following repeats

unsigned long EEtmA;          //time of last display update
unsigned long EEtmB;          //current time
const unsigned long EEtmVal = 2000; //delay to EEPROM write, mS
bool          EEtmReq;        //EEPROM write request

int dBint;                      // dB x4,  127 = 31.75dB
float dBdec;                    // dB in decimal
char dbuf[8];                   // formatting buffer

unsigned int PEbuf;   //buffer for PE_WR

void setup() {
  pinMode(SW_UP, INPUT_PULLUP);   //UP PB
  pinMode(SW_DN, INPUT_PULLUP);   //DOWN PB

  pinMode(LE , OUTPUT);   // control lines
  pinMode(CLK, OUTPUT);
  pinMode(DTA, OUTPUT);

  Wire.begin();
  Wire.setClock(400000L);
  oled.begin(&Adafruit128x64, I2C_ADDRESS);
  oled.setFont(ZevvPeep8x16); //non-proportional font

  oled.clear();
  oled.set2X();          //2X size font

  oled.setCursor(0,0);   //display fixed text
  oled.print(callSign);
  oled.setCursor(96,4);
  oled.print("dB");

  dBint=EEPROM.read(0x00);   //read stored dB value
  if(dBint>127) dBint = 127;
  
  display_dB();     //display stored value
  PE_WR();          //send value to attenuator
}

void loop() {
/* Action takes place on leading edge of the button push
   If PB is held the action repeats
*/
  SW_UP_R = !digitalRead(SW_UP);  //read switch input
  SW_DN_R = !digitalRead(SW_DN);
  if (SW_UP_R && !SW_UP_S) SW_UP_L = true; // latch set on leading edge
  if (SW_DN_R && !SW_DN_S) SW_DN_L = true;

    // switch store - for edge detection
  SW_UP_S = SW_UP_R;
  SW_DN_S = SW_DN_R;
  
  if(SW_UP_R && SW_DN_R){   // set dB to max if both PBs pushed
    dBint = 127;
    SW_UP_L=false;
    SW_DN_L=false;
    display_dB();
    PE_WR();          // send value to attenuator
  }

/* following causes one action per button push
   until the repeat action code reaches repeat time
*/
  if (SW_UP_L) {
      SW_UP_L = false;   // reset latch 
    if(dBint<127){ 
      ++dBint; 
      display_dB();
      PE_WR();          // send value to attenuator
    }  
  }

  if (SW_DN_L) {
      SW_DN_L = false;   // reset latch 
    if(dBint>0){
      --dBint;
      display_dB();
      PE_WR();          // send value to attenuator
    }
  }

/* repeat action code
   clears the stored switch image so edge detection will see a new edge
*/
  if((SW_UP_R&&!SW_DN_R) || (SW_DN_R&&!SW_UP_R)){  //one PB held on
    delay(10);
    if(++SWtmr>SWtmA){   //delay to repeat
      SWtmr=SWtmA-SWtmB; //set delay second to next repeat
      SW_UP_S=false;     //clear to allow repeat edge latch
      SW_DN_S=false;     //clear to allow repeat edge latch
    }
  }
  else{
    SWtmr=0;
  }

  if(!SW_UP_R && !SW_DN_R && EEtmReq){     //delay EEPROM write
    EEtmB = millis() & 0XFFFF;
    if(EEtmA>EEtmB){              //detect millis rollover
      EEtmB=EEtmB | 0X10000;
    }
    if((EEtmB-EEtmA) > EEtmVal){  //timed out
      EEtmReq=false;              //reset EE request
      EEPROM.update(0x00,dBint);  //write to EEPROM
    }
  }
}

void display_dB(){
  EEtmReq=true;               //request EEPROM write
  EEtmA = millis() & 0XFFFF;  //start time for EEPROM WR TIMER
  dBdec=dBint;                //move dB value to float
  dBdec=dBdec/4;              //divide to decimal value
  dtostrf(dBdec,5,2,dbuf);    //format to 2 decimal places
  oled.setCursor(0,4);
  oled.print(dbuf);
}

void PE_WR(){
  // PE43703 hardware setup
  // P/S set high by hardware DIP SW = OFF
  // all parallel inputs DI[6:0] low by hardware DIP SW's = ON
  // A0, A1, A2 low (If left open they float low to address 000)

  PEbuf=dBint;        //copy dB value to buffer
                      //Upper bits left at 0 for device address 0
  for (int i = 0; i <= 15; i++){
    digitalWrite(DTA, PEbuf & 0x01);  //output data bit
    digitalWrite(CLK,1);              //clock pulse
    digitalWrite(CLK,0);
    PEbuf=PEbuf>>1;                   //setup next bit
  }

    digitalWrite(LE,1);   //latch enable pulse
    digitalWrite(LE,0);
}
